----------------------------------------------------------------
--	Library
----------------------------------------------------------------
local BZ = LibStub("LibBabble-Zone-3.0"):GetLookupTable()
local BB = LibStub("LibBabble-Boss-3.0"):GetLookupTable()
----------------------------------------------------------------
--	Local Variables
----------------------------------------------------------------
local GetSpellInfo = GetSpellInfo
local fmt = string.format
local bit_band = bit.band
local MemberStatuses = {}

local debuffColorText = {
	[1] = "|cff3f8fff",
	[2] = "|cff008f00",
	[3] = "|cff8f5f00",
	[4] = "|cff8f00ff"
}

local debuffColorTable = {
	[1] = {r = 0.24, g = 0.56, b = 1, a = 1},
	[2] = {r = 0, g = 0.56, b = 0, a = 1},
	[3] = {r = 0.56, g = 0.37, b = 0, a = 1},
	[4] = {r = 0.56, g = 0, b = 1, a = 1},
}

local alertTable = {
	[1] = "alert_RaidDebuff_Icon1",
	[2] = "alert_RaidDebuff_Stack",
	[3] = "alert_RaidDebuff_Icon2",
}

local zone
local db
----------------------------------------------------------------
--	Localization
----------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("GridStatusRaidDebuff")
L:RegisterTranslations("enUS", function()
	return {
		["Raid Debuff"] = true,
		["Priority of %s"] = true,
		["RaidDebuff1"] = "Raid Debuff (Primary Icon)",
		["RaidDebuff2"] = "Raid Debuff (Stack Text)",
		["RaidDebuff3"] = "Raid Debuff (Secondary Icon)",
		["Option for %s"] = true,

		["Zone Name"] = true,
		["Enable %s"] = true,

	}
end)

L:RegisterTranslations("koKR", function()
	return {
		["Raid Debuff"] = "레이드 디버프",
		["Priority of %s"] = "%s의 우선순위",
		["RaidDebuff1"] = "레이드 디버프 (주아이콘)",
		["RaidDebuff2"] = "레이드 디버프 (중첩 문자)",
		["RaidDebuff3"] = "레이드 디버프 (부아이콘)",
		["Option for %s"] = "%s를 위한 설정",

		["Zone Name"] = "지역 이름",
		["Enable %s"] = "%s 활성화",
	}
end)

----------------------------------------------------------------
--	Module
----------------------------------------------------------------
GSRD = GridStatus:NewModule("GridStatusRaidDebuff")

local GSRD = GSRD
GSRD.menuName = L["Raid Debuff"]

GSRD.options = false

GSRD.defaultDB = {
	debug = false,

	alert_RaidDebuff_Icon1 = {
		text = L["RaidDebuff1"],
		desc = L["RaidDebuff1"],
		enable = true,
		color = { r = 1.0, g = 0.0, b = 0.0, a = 1 },
		priority = 98,
		range = false,
	},

	alert_RaidDebuff_Stack = {
		text = L["RaidDebuff2"],
		desc = L["RaidDebuff2"],
		enable = true,
		color = { r = 1.0, g = 0.0, b = 0.0, a = 1 },
		priority = 97,
		range = false,
	},

	alert_RaidDebuff_Icon2 = {
		text = L["RaidDebuff3"],
		desc = L["RaidDebuff3"],
		enable = true,
		color = { r = 1.0, g = 0.0, b = 0.0, a = 1 },
		priority = 96,
		range = false,
	},

	debuff_list = {
		[BZ["Karazhan"]] = {
			--Moroes
			[37066] = { --Garrote
				enable = true,
				order = 1,
				alert = 1,
			},
			--Maiden
			[29522] = { --Holy Fire
				enable = true,
				order = 4,
				debuffType = 1, --1 magic, 2 poison, 3 disease, 4 curse
				alert = 1,
			},
			[29511] = { --Repentance
				enable = true,
				order = 5,
				alert = 3,
			},
			--Opera : Bigbad wolf
			[30753] = { -- Red riding hood
				enable = true,
				order = 6,
				alert = 1,
			},
			--Illhoof
			[30115] = { --sacrifice
				enable = true,
				order = 7,
				alert = 1,
			},
			--Malche
			[30843]= { --Enfeeble
				enable = true,
				order = 8,
				alert = 1,
			},
		},
		[BZ["Zul'Aman"]] = {
			--Nalorakk
			[44955] = { --Mangle
				enable = true,
				order = 1,
				alert = 1,
			},
			--Akilzon
			[43657] = { --Electrical Storm
				enable = true,
				order = 2,
				alert = 1,
			},
			[43622] = { --Static Distruption
				enable = true,
				order = 3,
				alert = 3,
			},
			--Zanalai
			[43299] = { --Flame Buffet
				enable = true,
				order = 4,
				alert = 1,
				debuffType = 1,
			},
			--halazzi
			[43303] = { --Flame Shock
				enable = true,
				order = 5,
				alert = 1,
				debuffType = 1,
			},
			--hex lord
			[43613] = { --Cold Stare
				enable = true,
				order = 6,
				alert = 1,
				debuffType = 4,
			},
			[43501] = { --Siphon soul
				enable = true,
				order = 7,
				alert = 3,
			},
			--Zulzin
			[43093] = { --Throw
				enable = true,
				order = 8,
				alert = 1,
			},
			[43095] = { --Paralyze
				enable = true,
				order = 9,
				alert = 1,
			},
			[43150] = { --Rage
				enable = true,
				order = 10,
				alert = 1,
			},
		},
		[BZ["Serpentshrine Cavern"]] = {
			-- Trash
			[39042] = { --Rampent Infection
				enable = true,
				debuffType = 3,
				order = 1,
				alert = 1,
			},
			[39044] = { --Serpentshrine Parasite
				enable = true,
				order = 2,
				alert = 1,
			},
			--Hydross
			[38235] = { --Water Tomb
				enable = true,
				order = 3,
				alert = 1,
			},
			[38246] = { --Vile Sludge
				enable = true,
				order = 4,
				alert = 1,
			},
			--Morogrim
			[38049] = { --Watery Grave
				enable = true,
				order = 5,
				alert = 1,
			},
			--Leotheras
			[37676] = { --insidious whisper
				enable = true,
				order = 6,
				alert = 1,
			},
			[37640] = { --Whirl wind
				enable = true,
				order = 7,
				alert = 1,
			},
			[37749] = { --Madness
				enable = true,
				order = 8,
				alert = 1,
			},
			--Vashj
			[38280] = { --Static Charge
				enable = true,
				order = 9,
				alert = 1,
			},
		},
		[BZ["Tempest Keep"]] = {
			--Trash
			[35318] = { --Saw Blade
				enable = true,
				order = 1,
				alert = 1,
			},
			[37120] = { --Fragmentation Bomb
				enable = true,
				order = 2,
				alert = 1,
			},
			[37118] = { --Shell Shock
				enable = true,
				order = 3,
				alert = 3,
			},
			--Solarian
			[42783] = { --Wrath of the Astromancer
				enable = true,
				order = 4,
				alert = 1,
			},
			--Kaeltahas
			[37027] = { --Remote Toy
				enable = true,
				order = 5,
				alert = 3,
			},
			[36798] = { --Mind Control
				enable = true,
				order = 6,
				alert = 1,
			},
		},
		[BZ["Hyjal Summit"]] = {
			--Winterchill
			[31249] = { --Ice Bolt
				enable = true,
				order = 1,
				alert = 1,
			},
			--Aneteron
			[31306] = { --Carrion Swarm
				enable = true,
				order = 2,
				alert = 1,
			},
			[31298] = { --Sleep
				enable = true,
				order = 3,
				alert = 1,
			},
			--Azgalor
			[31347] = { --Doom
				enable = true,
				order = 4,
				alert = 3,
			},
			[31341] = { --Unquenchable Flames
				enable = true,
				order = 7,
				alert = 1,
			},
			--Achimonde
			[31944] = { --Doomfire
				enable = true,
				order = 8,
				alert = 3,
			},
			[31972] = { --Grip
				enable = true,
				order = 9,
				debuffType = 4,
				alert = 1,
			},
		},
		[BZ["Black Temple"]] = {
			--Najentus
			[39837] = { --Impaling Spine
				enable = true,
				order = 1,
				alert = 1,
			},
			--Terron
			[40239] = { --Incinerate
				enable = true,
				order = 4,
				alert = 1,
				debuffType = 1,
			},
			[40251] = { --Shadow of death
				enable = true,
				order = 5,
				alert = 3,
			},
			--Gurtogg
			[40604] = { --FelRage
				enable = true,
				order = 7,
				alert = 1,
			},
			[40481] = { --Acidic Wound
				enable = true,
				order = 8,
				alert = 2,
			},
			[40508] = { --Fel-Acid Breath
				enable = true,
				order = 9,
				alert = 3,
			},
			[42005] = { --bloodboil
				enable = true,
				order = 10,
				alert = 2,
			},
			--ROS
			[41303] = { --soulDrain
				enable = true,
				order = 11,
				alert = 1,

			},
			[41410] = { --Deaden
				enable = true,
				order = 12,
				alert = 1,
			},
			[41376] = { --Spite
				enable = true,
				order = 13,
				alert = 1,
			},
			[41377] = {
				delegater = 41376,
			},
			--Mother
			[40860] = { --Vile Beam
				enable = true,
				order = 14,
				alert = 3,
			},
			[41001] = { --Attraction
				enable = true,
				order = 15,
				alert = 1,
			},
			--Council
			[41485] = { --Deadly Poison
				enable = true,
				order = 16,
				alert = 1,
			},
			[41472] = { --Wrath
				enable = true,
				order = 17,
				alert = 3,
			},
			--Illiidan
			[41914] = { --Parasitic Shadowfiend
				enable = true,
				order = 18,
				alert = 1,
			},
			[41917] = {
				delegater = 41914,
			},
			[40585] = { --Dark Barrage
				enable = true,
				order = 18,
				alert = 1,
			},
			[41032] = { --Shear
				enable = true,
				order = 19,
				alert = 1,
			},
			[40932] = { --Flames
				enable = true,
				order = 20,
				alert = 1,
			},
		},
		[BZ["Sunwell Plateau"]] = {
			--Trash
			[46561] = { --Fear
				enable = true,
				order = 1,
				alert = 1,
				debuffType = 1,
			},
			[46562] = { --Mind Flay
				enable = true,
				order = 2,
				alert = 1,
			},
			[46266] = { --Burn Mana
				enable = true,
				order = 3,
				alert = 1,
				debuffType = 1,
			},
			[46557] = { --Slaying Shot
				enable = true,
				alert = 3,
				order = 4,
			},
			[46560] = { --Shadow Word: Pain
				enable = true,
				order = 5,
				alert = 1,
				debuffType = 1,
			},
			--Kalecgos
			[45032] = { --Curse of Boundless Agony
				enable = true,
				debuffIndex = 45032,
				order = 10,
				alert = 1,
				debuffType = 4,
			},
			[45034] = {
				delegater = 45032,
			},
			[45018] = { --Arcane Buffet
				enable = true,
				order = 12,
				alert = 2,
			},
			--Brutallus
			[46394] = { --Burn
				enable = true,
				order = 20,
				alert = 1,
			},
			[45150] = { --Meteor Slash
				enable = true,
				order = 21,
				alert = 2,
			},
			--Felmyst
			[45855] = { --Gas Nova
				enable = true,
				order = 30,
				alert = 1,
				debuffType = 1,
			},
			[45662] = { --Encaps
				enable = true,
				order = 31,
				alert = 1,
			},
			[45717] = { --Fog of Corruption
				enable = true,
				order = 32,
				alert = 1,
			},
			--Twins
			[45256] = { --Confounding Blow
				enable = true,
				order = 41,
				alert = 3,
			},
			[45333] = { --Conflagration
				enable = true,
				order = 42,
				alert = 1,
			},
			[46771] = { --Flame Sear
				enable = true,
				debuffIndex = 46771,
				order = 43,
				alert = 1,
			},
			[45270] = { --Shadowfury
				enable = true,
				debuffIndex = 45270,
				order = 44,
				alert = 3,
			},
			[45347] = { --Dark Touched
				enable = true,
				order = 45,
				alert = 2,
				color = { r = 0, g = 0, b = 0, a = 1},
			},
			[45348] = { --Fire Touched
				enable = true,
				order = 46,
				alert = 2,
			},
			--Muru
			[45996] = { --Darkness
				enable = true,
				order = 50,
				alert = 1,
			},
		},
	}
}

function GSRD:OnInitialize()
	self:PreLoader()
	self.super.OnInitialize(self)
	self:RegisterStatuses()
	self:CreateMenu()
end

--for get name and icon(Destination is when user played first time... he can see the debuff tooltip.. this is some problem)
function GSRD:PreLoader()
	for zone, cont in pairs(self.defaultDB["debuff_list"]) do
		for stNum, dInfo in pairs(cont) do
			if not dInfo.delegater then
				local link_name,_,link_icon = GetSpellInfo(stNum)
				dInfo.name = link_name
				dInfo.icon = link_icon
			end
		end
	end
end

function GSRD:OnEnable()
	self.debugging = self.db.profile.debug

	self:RegisterEvent("PLAYER_ENTERING_WORLD")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneCheck")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")

	db = self.db.profile.debuff_list
end

function GSRD:ZoneCheck()
	zone = GetRealZoneText()

	for i,k in pairs(MemberStatuses) do
		k = nil
		self:UpdateUnit(i)
	end


	if self.db.profile.debuff_list[zone] then
		if not self:IsEventRegistered("COMBAT_LOG_EVENT_UNFILTERED") then
			self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "EventHandler")
		end
	else
		if self:IsEventRegistered("COMBAT_LOG_EVENT_UNFILTERED") then
			self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		end
	end
end


function GSRD:PLAYER_ENTERING_WORLD()
	if not zone then
		self:ZoneCheck()
	else
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	end
end

function GSRD:RegisterStatuses()
	self:RegisterStatus("alert_RaidDebuff_Icon1",L["RaidDebuff1"],nil,false)
	self:RegisterStatus("alert_RaidDebuff_Icon2",L["RaidDebuff2"],nil,false)
	self:RegisterStatus("alert_RaidDebuff_Stack",L["RaidDebuff3"],nil,false)
end

function GSRD:UnregisterStatuses()
	self:UnregisterStatus("alert_RaidDebuff_Icon1",L["RaidDebuff1"],nil,false)
	self:UnregisterStatus("alert_RaidDebuff_Icon2",L["RaidDebuff2"],nil,false)
	self:UnregisterStatus("alert_RaidDebuff_Stack",L["RaidDebuff3"],nil,false)
end

function GSRD:CreateMenu()
	local db = self.db.profile.debuff_list
	local args = GridFrame.options.args.advanced.args

	args["Raid Debuff"] = {
		type = "group",
		name = L["Raid Debuff"],
		desc = L["Option for %s"]:format(L["Raid Debuff"]),
		args = {},
		order = 121,
	}
	args = args["Raid Debuff"].args

	for i = 1,3 do
		args["Alert"..i] = {
			type = "range",
			name = L["RaidDebuff"..i],
			desc = L["Priority of %s"]:format(L["RaidDebuff"..i]),
			min = 0,
			max = 99,
			step = 1,
			order = i,
			get = function() return self.db.profile[alertTable[i]].priority end,
			set = function(v) self.db.profile[alertTable[i]].priority = v end,
		}
	end


	for zone, dlist in pairs(db) do
		args[zone] = {
			type = "group",
			name = zone,
			desc = L["Option for %s"]:format(zone),
			args = {},
		}
		local zoneargs = args[zone].args

		for stNum, dInfo in pairs(dlist) do
			if not dInfo.delegater then
				local prefix = "|r"
				if dInfo.debuffType then
					prefix = debuffColorText[dInfo.debuffType]
				end

				zoneargs[stNum] = {
					type = "toggle",
					name = fmt("|T%s:0|t%s%s",dInfo.icon,prefix,dInfo.name),
					desc = L["RaidDebuff"..dInfo.alert],
					order = dInfo.order,
					get = function() return dInfo.enable end,
					set = function(v) dInfo.enable = v end,
				}
			end
		end
	end
end

function GSRD:Reset()
	self.super.Reset(self)
	self:UnregisterStatuses()
	self:RegisterStatuses()
end

function GSRD:EventHandler(ts, event, srcguid, srcname, srcflg, dstguid, dstname, dstflg, ...)
	if bit_band(dstflg, 0x00006868) == 0 then
		if
			event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_APPLIED_DOSE" or event == "SPELL_AURA_REMOVED_DOSE"
		then
			local spellId, name, _, auraType, count = select(1, ...)
			if auraType ~= "BUFF" then
				self:GainedDebuff(dstname, spellId, count or 1)
			end
		elseif
			event == "SPELL_AURA_REMOVED"
		then
			local spellId, name, _, auraType = select(1, ...)
			if auraType ~= "BUFF" then
				self:RemovedDebuff(dstname, spellId)
			end
		elseif
			event == "UNIT_DIED"
		then
			self:UnitDied(dstname)
		end
	end
end


function GSRD:GainedDebuff(uName, stNum, count)
	if db[zone] then
		local dInfo = db[zone][stNum]
		if dInfo then
			if dInfo.delegater then
				dInfo = db[zone][dInfo.delegater]
			end

			if dInfo.enable then
				if not MemberStatuses[uName] then MemberStatuses[uName] = {} end

				MemberStatuses[uName][stNum] = {
					alert = dInfo.alert,
					value = fmt("%d",count),
					color = dInfo.color or (dInfo.debuffType and debuffColorTable[dInfo.debuffType]) or {r = 1, g = 0, b = 0, a = 1},
					icon = dInfo.icon
				}

				self:UpdateUnit(uName)
			end
		end
	end
end

function GSRD:RemovedDebuff(uName, stNum)
	if MemberStatuses and MemberStatuses[uName] and MemberStatuses[uName][stNum] then
		MemberStatuses[uName][stNum] = nil
		self:UpdateUnit(uName)
	end
end

function GSRD:UnitDied(uName)
	if MemberStatuses and MemberStatuses[uName] then
		MemberStatuses[uName] = nil
		self:UpdateUnit(uName)
	end
end

function GSRD:UpdateUnit(uName)
	local hasStatus = { [1] = false, [2] = false, [3] = false }

	if MemberStatuses[uName] then
		if next(MemberStatuses[uName]) then
			for i,k in pairs(MemberStatuses[uName]) do
				local alertName = alertTable[k.alert]
				local settings = self.db.profile[alertName]
				self.core:SendStatusGained(
					uName, alertName, settings.priority, (settings.range and 40),
					k.color, k.value, nil, nil, k.icon
				)
				hasStatus[k.alert] = true
			end
		end
	end

	for i,k in ipairs(hasStatus) do
		if not k then
			self.core:SendStatusLost(uName, alertTable[i])
		end
	end
end